//`include "define.v"
//`default_nettype none

module ContestSys(
                // Clock
                input         CLOCK_50,
                
                // KEY
                input  [1: 0] KEY,
                
                // LEDG
                output [7:0] LEDG,
                
                // 7 SEGs
                //output [6:0] HEX0, HEX1, HEX2, HEX3,

                // UART interface connected to exStick
                inout [11:0] GPIO,
                
                // SDRAM Interface
                output [12:0] DRAM_ADDR,
                output [1:0] DRAM_BA,
                output DRAM_CAS_N, DRAM_RAS_N, DRAM_CLK,
                output DRAM_CKE, DRAM_CS_N, DRAM_WE_N,
                output [3:0] DRAM_DQM,
                inout [31:0] DRAM_DQ );

   wire  GPIO_RXD;     // GPIO[3]
   wire  GPIO_TXD;     // GPIO[5]
   wire  GPIO_FLUSH_X; // GPIO[7]
   wire  GPIO_INIT_X;  // GPIO[9]

   assign GPIO_RXD = GPIO[3];
   assign GPIO[5] = GPIO_TXD;
   assign GPIO[7] = GPIO_FLUSH_X;
   assign GPIO_INIT_X = GPIO[9];

   wire            sys_reset;
   wire            pll_locked;

   assign sys_reset = GPIO_INIT_X & KEY[0];
   assign GPIO_FLUSH_X = pll_locked & KEY[1];

        
   nios2_sys u0 ( .clk_clk(CLOCK_50),
                 .reset_reset_n(sys_reset),
                 .sdram_clk_clk(DRAM_CLK),
                 
                 .sdram_wire_addr(DRAM_ADDR),
                 .sdram_wire_ba(DRAM_BA),
                 .sdram_wire_cas_n(DRAM_CAS_N),
                 .sdram_wire_cke(DRAM_CKE),
                 .sdram_wire_cs_n(DRAM_CS_N),
                 .sdram_wire_dq(DRAM_DQ),
                 .sdram_wire_dqm(DRAM_DQM),
                 .sdram_wire_ras_n(DRAM_RAS_N),
                 .sdram_wire_we_n(DRAM_WE_N),
                 
                 .export_rx(GPIO_RXD),
                 .export_tx(GPIO_TXD),
                 //.export_state_init_done(init_done)

                  .locked_export(pll_locked)
                  
                 //.uart_rxd(rx_reg),
                 //.uart_txd(GPIO_TXD)
                 
                 );

   reg [25:0]      cnt_t;
   always@(posedge CLOCK_50) cnt_t <= cnt_t + 1;

   //assign LEDG[0] = init_done; // init done?
   assign LEDG[1] = 0; // dram calibration done: no signal for Altera DRAM IP core
   assign LEDG[2] = ~GPIO_RXD;
   assign LEDG[3] = ~GPIO_TXD;
   //assign LEDG[4] = mips_stall; // Dram busy = stall
   //assign LEDG[5] = mips_state[0]; // mips decode error: CORE_STAT[0]
   //assign LEDG[6] = mips_state[1]; // mips memory allignment error : CORE_STAT[1]
   assign LEDG[7] = cnt_t[25];
   
endmodule
/******************************************************************************************************/
