module contest_uart_avalon_interface(input             clock, reset_n,
                                     input [1:0]       avs_s1_address,
                                     input             avs_s1_read,
                                     input             avs_s1_write,
                                     input             avs_s1_chipselect,
                                     output reg [31:0] avs_s1_readdata,
                                     input [31:0]      avs_s1_writedata,
                                     
                                     input coe_externaluart_rx, 
                                     output coe_externaluart_tx
                                     );

`include "define.v"

   wire         read, write;

   assign read  = avs_s1_read  & avs_s1_chipselect;
   assign write = avs_s1_write & avs_s1_chipselect;
   
   reg [31:0]   rx_reg;
   wire [3:0]   uart_status;
   reg          rx_ready, recieve_overrun_error, transmit_overrun_error;
   wire        tx_ready;

   
   always@(posedge clock, negedge reset_n)
     begin
        if(!reset_n)
          avs_s1_readdata <= 0;
        else if(read)
          case(avs_s1_address)
            2'h0: avs_s1_readdata <= rx_reg;
            2'h2: avs_s1_readdata <= uart_status;
            default: avs_s1_readdata <= 0;
          endcase
     end


   assign uart_status = {transmit_overrun_error, tx_ready,
                         recieve_overrun_error, rx_ready};
   
   
   wire [31:0] rx_int_data;
   wire        rx_enable;

   always@(posedge clock, negedge reset_n)
     begin
        if(!reset_n)
          rx_reg <= 0;
        else
          if(rx_enable)
            rx_reg <= rx_int_data;
     end


   always@(posedge clock, negedge reset_n)
     begin
        if(!reset_n)
          rx_ready <= 0;
        else
          begin
             if(rx_enable)
               rx_ready <= 1'b1;
             else if( (avs_s1_address == 2'h0) && (read==1'b1) )
               rx_ready <= 1'b0;
          end
     end

   always@(posedge clock, negedge reset_n)
     begin
        if(!reset_n)
          recieve_overrun_error <= 0;
        else
          begin
             if((rx_enable==1'b1) && (rx_ready==1'b1))
               recieve_overrun_error <= 1'b1;
          end
     end
   
   rx_char2int rx_char2int( .clock(clock), .reset_n(reset_n),
                .rx(coe_externaluart_rx),
                .data(rx_int_data),
                .enable(rx_enable) );



   wire        tx_we;
   assign tx_we = (avs_s1_address == 2'h1) ? write : 0;

   always@(posedge clock, negedge reset_n)
     begin
        if(!reset_n)
          transmit_overrun_error <= 0;
        else
          begin
             if((tx_we==1'b1) && (tx_ready==1'b0) )
               transmit_overrun_error <= 1'b1;
          end
     end

   
   UartTx uarttx( .CLK(clock), .RST_X(reset_n), .DATA(avs_s1_writedata[7:0]), 
                  .WE(write), 
                  .TXD(coe_externaluart_tx), .READY(tx_ready) );
   
   
endmodule
   
