/******************************************************************************************************/
/* The 1st IPSJ SIG-ARC High-Performance Processor Design Contest                     Version 2013-09 */
/* From MieruPC 2010: Top Level Module                                            ArchLab. TOKYO TECH */
/******************************************************************************************************/


/******************************************************************************************************/
module UartTx(CLK, RST_X, DATA, WE, TXD, READY);
    input       CLK, RST_X, WE;
    input [7:0] DATA;
    output reg  TXD, READY;

    reg [8:0]   cmd;
    reg [11:0]  waitnum;
    reg [3:0]   cnt;

    always @(posedge CLK or negedge RST_X) begin
        if(!RST_X) begin
            TXD       <= 1'b1;
            READY     <= 1'b1;
            cmd       <= 9'h1ff;
            waitnum   <= 0;
            cnt       <= 0;
        end else if( READY ) begin
            TXD       <= 1'b1;
            waitnum   <= 0;
            if( WE )begin
                READY <= 1'b0;
                cmd   <= {DATA, 1'b0};
                cnt   <= 10;
            end
        end else if( waitnum >= `SERIAL_WCNT ) begin
            TXD       <= cmd[0];
            READY     <= (cnt == 1);
            cmd       <= {1'b1, cmd[8:1]};
            waitnum   <= 1;
            cnt       <= cnt - 1;
        end else begin
            waitnum   <= waitnum + 1;
        end
    end
endmodule // UartTx


/******************************************************************************************************/
/* RS232C serial controller (deserializer):                                                           */
/******************************************************************************************************/
`define SS_SER_WAIT  'd0         // RS232C deserializer, State WAIT
`define SS_SER_RCV0  'd1         // RS232C deserializer, State Receive 0th bit
                                 // States Receive 1st bit to 7th bit are not used
`define SS_SER_DONE  'd9         // RS232C deserializer, State DONE
/******************************************************************************************************/
module serialc(CLK, RST_X, RXD, DATA, EN);
    input          CLK, RST_X, RXD; // clock, reset, RS232C input
    output [7:0]   DATA;            // 8bit output data
    output reg     EN;              // 8bit output data enable

    reg    [7:0]   DATA;
    reg    [3:0]   stage;
    reg    [12:0]  cnt;             // counter to latch D0, D1, ..., D7
    reg    [11:0]  cnt_start;       // counter to detect the Start Bit
    wire   [12:0]  waitcnt;

   assign waitcnt = `SERIAL_WCNT;

   reg             rxd_reg;

   always @(posedge CLK ) rxd_reg <= RXD;

   always @(posedge CLK or negedge RST_X)
      if (!RST_X) cnt_start <= 0;
      else        cnt_start <= (rxd_reg) ? 0 : cnt_start + 1;

   
    always @(posedge CLK or negedge RST_X)
      if(!RST_X) begin
          EN     <= 0;
          stage  <= `SS_SER_WAIT;
          cnt    <= 1;
          DATA   <= 0;
      end else if (stage == `SS_SER_WAIT) begin // detect the Start Bit
          EN <= 0;
          stage <= (cnt_start == (waitcnt >> 1)) ? `SS_SER_RCV0 : stage;
      end else begin
          if (cnt != waitcnt) begin
              cnt <= cnt + 1;
              EN <= 0;
          end else begin               // receive 1bit data
              stage  <= (stage == `SS_SER_DONE) ? `SS_SER_WAIT : stage + 1;
              EN     <= (stage == 8)  ? 1 : 0;
              DATA   <= {rxd_reg, DATA[7:1]};
              cnt <= 1;
          end
      end
    
 
endmodule
/******************************************************************************************************/

module rx_char2int( input clock, reset_n,
                    input rx,
                    output reg [31:0] data,
                    output reg enable );

   wire [7:0]       char_data;
   wire             char_enable;
   
   serialc serialc(.CLK(clock), .RST_X(reset_n), .RXD(rx), .DATA(char_data), .EN(char_enable));
   
   always@(posedge clock or negedge reset_n )
     begin
        if( !reset_n )
          data <= 0;
        else
          begin
             if( char_enable == 1'b1 )
               data <= {char_data, data[31:8]};
          end
     end

   reg [1:0] address;

   always@(posedge clock or negedge reset_n )
     begin
        if( !reset_n)
          address <= 0;
        else
          begin
             if( char_enable == 1'b1 )
               address <= address + 1;
          end
     end
   
   always@(posedge clock or negedge reset_n )
     begin
        if( !reset_n)
          enable <= 0;
        else
          begin
             if( (char_enable == 1'b1) && (address==2'h3) )
               enable <= 1'b1;
             else
               enable <= 1'b0;
          end
     end


   
endmodule
